@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,16 +21,59 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * Portions bsed upon the file crypto/buffer/bss_sock.c 
  * Original copyright notice appears below
  *
- * $Id: pssl.cxx,v 1.27 2001/10/31 01:31:26 robertj Exp $
+ * $Id: pssl.cxx,v 1.41 2005/11/30 12:47:41 csoutheren Exp $
  * $Log: pssl.cxx,v $
+ * Revision 1.41  2005/11/30 12:47:41  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.40  2005/10/17 01:25:05  csoutheren
+ * Added check for ssl with const argumetns
+ *
+ * Revision 1.39  2004/04/09 06:52:17  rjongbloed
+ * Removed #pargma linker command for /delayload of DLL as documentations sais that
+ *   you cannot do this.
+ *
+ * Revision 1.38  2004/02/23 23:52:20  csoutheren
+ * Added pragmas to avoid every Windows application needing to include libs explicitly
+ *
+ * Revision 1.37  2004/02/22 01:57:37  ykiryanov
+ * Put a fix for a compiler choke on BeOS when calling macro d2i_DHparams_bio in PSSLDiffieHellman::Load. Fast on Monday.
+ *
+ * Revision 1.36  2003/04/16 08:00:19  robertj
+ * Windoes psuedo autoconf support
+ *
+ * Revision 1.35  2002/11/06 22:47:25  robertj
+ * Fixed header comment (copyright etc)
+ *
+ * Revision 1.34  2002/06/07 02:55:23  robertj
+ * Fixed GNU warning
+ *
+ * Revision 1.33  2002/04/02 16:59:35  robertj
+ * Fixed GNU warning
+ *
+ * Revision 1.32  2002/04/02 16:17:28  robertj
+ * Fixed bug where returned TRUE when read count 0 or write count not len
+ *   which is not according to Read()/Write() semantics. Could cause high CPU
+ *   loops when sockets shutdown.
+ *
+ * Revision 1.31  2002/03/28 07:26:25  robertj
+ * Added Diffie-Hellman parameters wrapper class.
+ *
+ * Revision 1.30  2001/12/13 09:15:41  robertj
+ * Added function to get private key as ray DER binary data or as base64 string.
+ *
+ * Revision 1.29  2001/12/06 04:06:03  robertj
+ * Removed "Win32 SSL xxx" build configurations in favour of system
+ *   environment variables to select optional libraries.
+ *
+ * Revision 1.28  2001/12/04 02:59:18  robertj
+ * Fixed problem on platforms where a pointer is not the same size as an int.
+ *
  * Revision 1.27  2001/10/31 01:31:26  robertj
  * Added enhancements for saving/loading/creating certificates and keys.
  *
@@ -162,6 +205,7 @@
 #include <ptclib/pssl.h>
 #include <ptclib/mime.h>
 
+#if P_SSL
 
 #define USE_SOCKETS
 
@@ -169,12 +213,21 @@ extern "C" {
 
 #include <openssl/ssl.h>
 #include <openssl/err.h>
-#include <openssl/crypto.h>
-#include <openssl/buffer.h>
 #include <openssl/rand.h>
 
 };
 
+
+#ifdef _MSC_VER
+
+#pragma comment(lib, P_SSL_LIB1)
+#pragma comment(lib, P_SSL_LIB2)
+
+#endif
+
+
+///////////////////////////////////////////////////////////////////////////////
+
 PARRAY(PSSLMutexArrayBase, PMutex);
 class PSSLMutexArray : public PSSLMutexArrayBase
 {
@@ -250,14 +303,22 @@ PSSLPrivateKey::PSSLPrivateKey(const PFilePath & keyFile, PSSLFileTypes fileType
 
 PSSLPrivateKey::PSSLPrivateKey(const BYTE * keyData, PINDEX keySize)
 {
+#if P_SSL_USE_CONST
+  key = d2i_AutoPrivateKey(NULL, &keyData, keySize);
+#else
   key = d2i_AutoPrivateKey(NULL, (BYTE **)&keyData, keySize);
+#endif
 }
 
 
 PSSLPrivateKey::PSSLPrivateKey(const PBYTEArray & keyData)
 {
   const BYTE * keyPtr = keyData;
+#if P_SSL_USE_CONST
+  key = d2i_AutoPrivateKey(NULL, &keyPtr, keyData.GetSize());
+#else
   key = d2i_AutoPrivateKey(NULL, (BYTE **)&keyPtr, keyData.GetSize());
+#endif
 }
 
 
@@ -311,6 +372,25 @@ BOOL PSSLPrivateKey::Create(unsigned modulus,
 }
 
 
+PBYTEArray PSSLPrivateKey::GetData() const
+{
+  PBYTEArray data;
+
+  if (key != NULL) {
+    BYTE * keyPtr = data.GetPointer(i2d_PrivateKey(key, NULL));
+    i2d_PrivateKey(key, &keyPtr);
+  }
+
+  return data;
+}
+
+
+PString PSSLPrivateKey::AsString() const
+{
+  return PBase64::Encode(GetData());
+}
+
+
 BOOL PSSLPrivateKey::Load(const PFilePath & keyFile, PSSLFileTypes fileType)
 {
   if (key != NULL) {
@@ -406,14 +486,22 @@ PSSLCertificate::PSSLCertificate(const PFilePath & certFile, PSSLFileTypes fileT
 
 PSSLCertificate::PSSLCertificate(const BYTE * certData, PINDEX certSize)
 {
+#if P_SSL_USE_CONST
+  certificate = d2i_X509(NULL, &certData, certSize);
+#else
   certificate = d2i_X509(NULL, (unsigned char **)&certData, certSize);
+#endif
 }
 
 
 PSSLCertificate::PSSLCertificate(const PBYTEArray & certData)
 {
   const BYTE * certPtr = certData;
+#if P_SSL_USE_CONST
+  certificate = d2i_X509(NULL, &certPtr, certData.GetSize());
+#else
   certificate = d2i_X509(NULL, (unsigned char **)&certPtr, certData.GetSize());
+#endif
 }
 
 
@@ -423,7 +511,11 @@ PSSLCertificate::PSSLCertificate(const PString & certStr)
   PBase64::Decode(certStr, certData);
   if (certData.GetSize() > 0) {
     const BYTE * certPtr = certData;
+#if P_SSL_USE_CONST
+    certificate = d2i_X509(NULL, &certPtr, certData.GetSize());
+#else
     certificate = d2i_X509(NULL, (unsigned char **)&certPtr, certData.GetSize());
+#endif
   }
   else
     certificate = NULL;
@@ -623,6 +715,115 @@ BOOL PSSLCertificate::Save(const PFilePath & certFile, BOOL append, PSSLFileType
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+
+PSSLDiffieHellman::PSSLDiffieHellman()
+{
+  dh = NULL;
+}
+
+
+PSSLDiffieHellman::PSSLDiffieHellman(const PFilePath & dhFile,
+                                     PSSLFileTypes fileType)
+{
+  dh = NULL;
+  Load(dhFile, fileType);
+}
+
+
+PSSLDiffieHellman::PSSLDiffieHellman(const BYTE * pData, PINDEX pSize,
+                                     const BYTE * gData, PINDEX gSize)
+{
+  dh = DH_new();
+  if (dh == NULL)
+    return;
+
+  dh->p = BN_bin2bn(pData, pSize, NULL);
+  dh->g = BN_bin2bn(gData, gSize, NULL);
+  if (dh->p != NULL && dh->g != NULL)
+    return;
+
+  DH_free(dh);
+  dh = NULL;
+}
+
+
+PSSLDiffieHellman::PSSLDiffieHellman(const PSSLDiffieHellman & diffie)
+{
+  dh = diffie.dh;
+}
+
+
+PSSLDiffieHellman & PSSLDiffieHellman::operator=(const PSSLDiffieHellman & diffie)
+{
+  if (dh != NULL)
+    DH_free(dh);
+  dh = diffie.dh;
+  return *this;
+}
+
+
+PSSLDiffieHellman::~PSSLDiffieHellman()
+{
+  if (dh != NULL)
+    DH_free(dh);
+}
+
+#ifdef __BEOS__
+// 2/21/04 Yuri Kiryanov - fix for compiler choke on BeOS for usage of
+// SSL function d2i_DHparams_bio below in PSSLDiffieHellman::Load
+#undef  d2i_DHparams_bio
+#define d2i_DHparams_bio(bp,x) \
+ (DH *)ASN1_d2i_bio( \
+         (char *(*)(...))(void *)DH_new, \
+         (char *(*)(...))(void *)d2i_DHparams, \
+         (bp), \
+         (unsigned char **)(x) \
+)
+#endif
+
+BOOL PSSLDiffieHellman::Load(const PFilePath & dhFile,
+                             PSSLFileTypes fileType)
+{
+  if (dh != NULL) {
+    DH_free(dh);
+    dh = NULL;
+  }
+
+  PSSL_BIO in;
+  if (!in.OpenRead(dhFile)) {
+    SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE,ERR_R_SYS_LIB);
+    return FALSE;
+  }
+
+  if (fileType == PSSLFileTypeDEFAULT)
+    fileType = dhFile.GetType() == ".pem" ? PSSLFileTypePEM : PSSLFileTypeASN1;
+
+  switch (fileType) {
+    case PSSLFileTypeASN1 :
+      dh = d2i_DHparams_bio(in, NULL);
+      if (dh != NULL)
+        return TRUE;
+
+      SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE, ERR_R_ASN1_LIB);
+      break;
+
+    case PSSLFileTypePEM :
+      dh = PEM_read_bio_DHparams(in, NULL, NULL, NULL);
+      if (dh != NULL)
+        return TRUE;
+
+      SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE, ERR_R_PEM_LIB);
+      break;
+
+    default :
+      SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE,SSL_R_BAD_SSL_FILETYPE);
+  }
+
+  return FALSE;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 static void LockingCallback(int mode, int n, const char * /*file*/, int /*line*/)
@@ -664,8 +865,6 @@ static void PSSLAssert(const char * msg)
 
 PSSLContext::PSSLContext(const void * sessionId, PINDEX idSize)
 {
-  PINDEX i;
-
   static PMutex InitialisationMutex;
   InitialisationMutex.Wait();
 
@@ -676,7 +875,7 @@ PSSLContext::PSSLContext(const void * sessionId, PINDEX idSize)
 
     // Seed the random number generator
     BYTE seed[128];
-    for (i = 0; i < sizeof(seed); i++)
+    for (size_t i = 0; i < sizeof(seed); i++)
       seed[i] = (BYTE)rand();
     RAND_seed(seed, sizeof(seed));
 
@@ -754,6 +953,12 @@ BOOL PSSLContext::UsePrivateKey(const PSSLPrivateKey & key)
 }
 
 
+BOOL PSSLContext::UseDiffieHellman(const PSSLDiffieHellman & dh)
+{
+  return SSL_CTX_set_tmp_dh(context, (dh_st *)dh) > 0;
+}
+
+
 BOOL PSSLContext::SetCipherList(const PString & ciphers)
 {
   if (ciphers.IsEmpty())
@@ -823,8 +1028,8 @@ BOOL PSSLChannel::Read(void * buf, PINDEX len)
 
     int readResult = SSL_read(ssl, (char *)buf, len);
     lastReadCount = readResult;
-    returnValue = readResult >= 0;
-    if (!returnValue && GetErrorCode(LastReadError) == NoError)
+    returnValue = readResult > 0;
+    if (readResult < 0 && GetErrorCode(LastReadError) == NoError)
       ConvertOSError(-1, LastReadError);
   }
 
@@ -851,8 +1056,8 @@ BOOL PSSLChannel::Write(const void * buf, PINDEX len)
 
     int writeResult = SSL_write(ssl, (const char *)buf, len);
     lastWriteCount = writeResult;
-    returnValue = writeResult >= 0;
-    if (!returnValue && GetErrorCode(LastWriteError) == NoError)
+    returnValue = lastWriteCount >= len;
+    if (writeResult < 0 && GetErrorCode(LastWriteError) == NoError)
       ConvertOSError(-1, LastWriteError);
   }
 
@@ -984,7 +1189,7 @@ BOOL PSSLChannel::RawSSLRead(void * buf, PINDEX & len)
 //
 
 
-#define	PSSLSOCKET(bio)			((PSSLChannel *)(bio->num))
+#define PSSLCHANNEL(bio)      ((PSSLChannel *)(bio->ptr))
 
 extern "C" {
 
@@ -998,8 +1203,8 @@ typedef long (*lfptr)();
 static int Psock_new(BIO * bio)
 {
   bio->init     = 0;
-  bio->num      = 0;    // this is really (PSSLChannel *), not int
-  bio->ptr      = NULL;
+  bio->num      = 0;
+  bio->ptr      = NULL;    // this is really (PSSLChannel *)
   bio->flags    = 0;
 
   return(1);
@@ -1013,8 +1218,8 @@ static int Psock_free(BIO * bio)
 
   if (bio->shutdown) {
     if (bio->init) {
-      PSSLSOCKET(bio)->Shutdown(PSocket::ShutdownReadAndWrite);
-      PSSLSOCKET(bio)->Close();
+      PSSLCHANNEL(bio)->Shutdown(PSocket::ShutdownReadAndWrite);
+      PSSLCHANNEL(bio)->Close();
     }
     bio->init  = 0;
     bio->flags = 0;
@@ -1023,48 +1228,22 @@ static int Psock_free(BIO * bio)
 }
 
 
-static long Psock_ctrl(BIO * bio, int cmd, long num, void * ptr)
+static long Psock_ctrl(BIO * bio, int cmd, long num, void * /*ptr*/)
 {
-  long ret = 1;
-  int *ip;
-
   switch (cmd) {
-    case BIO_C_SET_FD:
-      Psock_free(bio);
-      bio->num      = *((int *)ptr);
-      bio->shutdown = (int)num;
-      bio->init     = 1;
-      break;
-
-    case BIO_C_GET_FD:
-      if (bio->init) {
-	ip = (int *)ptr;
-	if (ip != NULL)
-          *ip = bio->num;
-	ret = bio->num;
-      }
-      else
-        ret = -1;
-      break;
-
-    case BIO_CTRL_GET_CLOSE:
-      ret = bio->shutdown;
-      break;
-
     case BIO_CTRL_SET_CLOSE:
       bio->shutdown = (int)num;
-      break;
-
-    case BIO_CTRL_DUP:
-    case BIO_CTRL_FLUSH:
       return 1;
 
-    // Other BIO commands, return 0
-    default:
-      return 0;
+    case BIO_CTRL_GET_CLOSE:
+      return bio->shutdown;
+
+    case BIO_CTRL_FLUSH:
+      return 1;
   }
 
-  return ret;
+  // Other BIO commands, return 0
+  return 0;
 }
 
 
@@ -1077,10 +1256,10 @@ static int Psock_read(BIO * bio, char * out, int outl)
 
   // Skip over the polymorphic read, want to do real one
   PINDEX len = outl;
-  if (PSSLSOCKET(bio)->RawSSLRead(out, len))
+  if (PSSLCHANNEL(bio)->RawSSLRead(out, len))
     return len;
 
-  switch (PSSLSOCKET(bio)->GetErrorCode(PChannel::LastReadError)) {
+  switch (PSSLCHANNEL(bio)->GetErrorCode(PChannel::LastReadError)) {
     case PChannel::Interrupted :
     case PChannel::Timeout :
       BIO_set_retry_read(bio);
@@ -1102,10 +1281,10 @@ static int Psock_write(BIO * bio, const char * in, int inl)
   BIO_clear_retry_flags(bio);
 
   // Skip over the polymorphic write, want to do real one
-  if (PSSLSOCKET(bio)->PIndirectChannel::Write(in, inl))
-    return PSSLSOCKET(bio)->GetLastWriteCount();
+  if (PSSLCHANNEL(bio)->PIndirectChannel::Write(in, inl))
+    return PSSLCHANNEL(bio)->GetLastWriteCount();
 
-  switch (PSSLSOCKET(bio)->GetErrorCode(PChannel::LastWriteError)) {
+  switch (PSSLCHANNEL(bio)->GetErrorCode(PChannel::LastWriteError)) {
     case PChannel::Interrupted :
     case PChannel::Timeout :
       BIO_set_retry_write(bio);
@@ -1135,7 +1314,7 @@ static int Psock_puts(BIO * bio, const char * str)
 static BIO_METHOD methods_Psock =
 {
   BIO_TYPE_SOCKET,
-  "ptlib socket",
+  "PTLib-PSSLChannel",
 #if (OPENSSL_VERSION_NUMBER < 0x00906000)
   (ifptr)Psock_write,
   (ifptr)Psock_read,
@@ -1164,21 +1343,11 @@ BOOL PSSLChannel::OnOpen()
     return FALSE;
   }
 
+  // "Open" then bio
+  bio->ptr  = this;
+  bio->init = 1;
+
   SSL_set_bio(ssl, bio, bio);
-
-  BIO_set_fd(bio, (int)this, BIO_NOCLOSE);
-
-/*
-  if (ssl->rbio != NULL)
-    BIO_free((bio_st *)ssl->rbio);
-
-  if ((ssl->wbio != NULL) && (ssl->wbio != ssl->rbio))
-    BIO_free((bio_st *)ssl->wbio);
-
-  ssl->rbio = bio;
-  ssl->wbio = bio;
-  ret = 1;
-*/
   return TRUE;
 }
 
@@ -1220,7 +1389,7 @@ int verify_callback(int ok, X509 * xs, X509 * xi, int depth, int error)
 
   if (!ok) {
     PError << "verify error:num=" << error << " " <<
-			X509_cert_verify_error_string(error) << endl;
+    X509_cert_verify_error_string(error) << endl;
     if (verify_depth <= depth) {
       ok=1;
       verify_error=VERIFY_OK;
@@ -1237,3 +1406,7 @@ int verify_callback(int ok, X509 * xs, X509 * xi, int depth, int error)
 
 #endif
 
+#endif // P_SSL
+
+
+// End of file ////////////////////////////////////////////////////////////////
